/* MuSE - Multiple Streaming Engine
 * Copyright (C) 2000-2002 Denis Roio aka jaromil <jaromil@dyne.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <iostream.h>
#include <config.h>

/* headers needed for config stuff */
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
/* ------------------------------- */

#include <generic.h>
#include <gui_muse.h>
#include <gui_pixmaps.h>
#include <jmixer.h>
#include <jsync.h>
#include <jutils.h>
#include <FL/fl_ask.H>
#include <FL/Fl_File_Chooser.H>
#include <FL/filename.H>

static Stream_mixer *mixer;
static Amuse *amuse;

/* where to store volumes for the talk func */
static float store_vol[MAX_CHANNELS];
static char pick_file_dir[1024];

static char *home;

/* resolves the bitrate widget choice */
int solve_bps(int bps_choice) {
  int bps = 0;

  switch(bps_choice) {
  case 0:
    bps = 8;
    break;
  case 1:
    bps = 16;
    break;
  case 2:
    bps = 24;
    break;
  case 3:
    bps = 32;
    break;
  case 4:
    bps = 56;
    break;
  case 5:
    bps = 64;
    break;
  case 6:
    bps = 96;
    break;
  case 7:
    bps = 128;
    break;
  }
  return(bps);
}

int solve_frequency(int freq) {
  int f = 0;
  switch(freq) {
  case 0:
    f = freq;
    break;
  case 1:
    f = 11000;
    break;
  case 2:
    f = 16000;
    break;
  case 3:
    f = 22050;
    break;
  case 4:
    f = 44100;
    break;
  }
  return(f);
}

/* ================================== config file stuff */

void save_conf() {
  char *line;
  char config[MAX_PATH_SIZE];
  char temp[256];
  FILE *fd;
  
  home = getenv("HOME");
  sprintf(config,"%s/.muse/muserc",home);

  fd = fopen(config,"r+");
  if(fd==NULL) {
    warning("cannot save configuration defaults to %s",config);
    return;
  }

  while(strncmp(temp,"[server]",8)!=0) {
    line = fgets(temp,256,fd);
    if(line==NULL) {
      warning("configuration file broken: missing [server] section");
      return;
    }
  }

  /* 1 - server address */
  sprintf(temp,"%s\n",amuse->ip_input->value());
  fputs(temp,fd);

  /* 2 - port number */
  sprintf(temp,"%s\n",amuse->port_input->value());
  fputs(temp,fd);

  /* 3 - mount point */
  sprintf(temp,"%s\n",amuse->mount_input->value());
  fputs(temp,fd);

  /* 4 - bps TODO */
  sprintf(temp,"%u\n",amuse->bitrate_choice->value());
  fputs(temp,fd);
  
  /* 5 - metadata: name */
  sprintf(temp,"%s\n",amuse->name_input->value());
  fputs(temp,fd);

  /* 6 - metadata: url */
  sprintf(temp,"%s\n",amuse->url_input->value());
  fputs(temp,fd);
  
  /* 7 - metadata: description */
  sprintf(temp,"%s\n",amuse->description_input->value());
  fputs(temp,fd);

  fflush(fd);
  fclose(fd);
}

void load_conf(FILE *fd) { 
  char *line = NULL, *p;
  char temp[256];
  
  while(strncmp(temp,"[server]",8)!=0) {
    line = fgets(temp,256,fd);
    if(line==NULL) break;
  }
  
  /* we don't have a server entry already */  
  if(line==NULL) return;
  
  /* 1 - server address */
  line = fgets(temp,256,fd);
  if(line==NULL) return;
  p = temp; while(*p!='\n') p++; *p = '\0';
  amuse->ip_input->value(temp);

  /* 2 - port number */
  line = fgets(temp,256,fd);
  if(line==NULL) return;
  p = temp; while(*p!='\n') p++; *p = '\0';
  amuse->port_input->value(temp);

  /* 3 - mount point */
  line = fgets(temp,256,fd);
  if(line==NULL) return;
  p = temp; while(*p!='\n') p++; *p = '\0';
  amuse->mount_input->value(temp);

  /* 4 - bps TODO */
  line = fgets(temp,256,fd);
  if(line==NULL) return;
  p = temp; while(*p!='\n') p++; *p = '\0';
  amuse->bitrate_choice->value(atoi(temp));

  /* 5 - metadata: name */
  line = fgets(temp,256,fd);
  if(line==NULL) return;
  p = temp; while(*p!='\n') p++; *p = '\0';
  amuse->name_input->value(temp);
  
  /* 6 - metadata: url */
  line = fgets(temp,256,fd);
  if(line==NULL) return;
  p = temp; while(*p!='\n') p++; *p = '\0';
  amuse->url_input->value(temp);

  /* 7 - metadata: description */
  line = fgets(temp,256,fd);
  if(line==NULL) return;
  p = temp; while(*p!='\n') p++; *p = '\0';
  amuse->description_input->value(temp);
  
}

void create_conf() {
  FILE *fd;
  char temp[MAX_PATH_SIZE];
  
  sprintf(temp,"%s/.muse",home);
  mkdir(temp,0744);  
  sprintf(temp,"%s/.muse/muserc",home);
  fd = fopen(temp,"w");

  if(fd==NULL) {
    error("i don't reach to create a new default config file in %s",temp);
    act("bailing out :(");
    exit(1);
  }

  act("a new default config file has been created in %s",temp);
  fputs("# Config file generated automaticly by MuSE\n",fd);
  fputs("# the lines below are position sensitive\n",fd);
  fputs("# if you edit by hand here, do it wise ;^)\n",fd);
  sprintf(temp,"[version]\n%s\n",VERSION);
  fputs(temp,fd);
  fputs("[server]\n",fd);
  fclose(fd);

}

void check_conf() {
  FILE *fd;
  char *home;
  char temp[MAX_PATH_SIZE];
  
  home = getenv("HOME");

  sprintf(temp,"%s/.muse/muserc",home);
  fd = fopen(temp,"r");
  if(fd != NULL) {

    /* here we read what we need from muse.conf */    
    load_conf(fd);

    fclose(fd);
  } else {
    notice("This seems the first time you run MuSE");
    create_conf();
  }
}

/* ============================== end config file stuff */

void init_callbacks(Stream_mixer *mix, Amuse *gui) {
  mixer = mix;
  amuse = gui;
  mix->register_gui(gui);

  home = getenv("HOME");
  strncpy(pick_file_dir,home,MAX_PATH_SIZE);
  check_conf();
}

void quit_mixer() {
  mixer->quit = true;
  mixer->signal();
}

void signal_mixer() {
  mixer->signal();
}

void set_channel(Fl_Widget *o, void *ch, int song){
  int tmpchan = *(int*)ch;
  int result;
  /* playlist starts from 1   Fl browser value starts from 1 */ 
  result = mixer->set_channel(tmpchan,song);
  switch (result) {
  case 0:
    amuse->status_main->value("unable to open the bitstream");
    break;
  case 1:
    /* bitstream opened: is seekable */
      amuse->chan[tmpchan]->playlist_browser->value(song);
      amuse->chan[tmpchan]->timeline_slider->activate();
      amuse->chan[tmpchan]->timeline_slider->value(0.0);     
      amuse->chan[tmpchan]->time->value("00:00:00");
      amuse->chan[tmpchan]->name->value(amuse->chan[tmpchan]->playlist_browser->text(song)); 
      break;
  case 2:
    /* bitstream opened: is not seekable */
    amuse->chan[tmpchan]->playlist_browser->value(song);
    amuse->chan[tmpchan]->timeline_slider->deactivate();     
    amuse->chan[tmpchan]->time->value("stream");
    amuse->chan[tmpchan]->name->value(amuse->chan[tmpchan]->playlist_browser->text(song));
    break;
  }
}

void cb_set_channel(Fl_Widget *o, void *ch) {
  int song = ((Fl_Select_Browser *)o)->value();
  set_channel(o, ch, song);
}

void cb_set_channel_plus(Fl_Widget *o, void *ch) {  
  int tmpchan = *(int*)ch;
   int song = amuse->chan[tmpchan]->playlist_browser->value();
   song++;
   if (song <= amuse->chan[tmpchan]->playlist_browser->size())
     set_channel(o, ch, song);
}

void cb_set_channel_minus(Fl_Widget *o, void *ch) {
  int tmpchan = *(int*)ch;  
  int song = amuse->chan[tmpchan]->playlist_browser->value();
  song--;
  if (song > 0)
    set_channel(o, ch, song);
}

void cb_play_channel(Fl_Widget *o, void *ch) {
  mixer->play_channel(*(int*)ch);
}

void cb_stop_channel(Fl_Widget *o, void *ch) {
  int tmpchan = *(int*)ch;
  mixer->stop_channel(tmpchan);
  amuse->chan[tmpchan]->timeline_slider->value(0.0);
  amuse->chan[tmpchan]->time->value("00:00:00");
}

void cb_pause_channel(Fl_Widget *o, void *ch) {
  mixer->pause_channel(*(int*)ch);
}

void cb_set_volume(Fl_Widget *o, void *ch) {
  float vol = ((Fl_Slider *)o)->value();
  mixer->set_volume(*(int*)ch,vol);
}

void cb_set_speed(Fl_Widget *o, void *ch) {
  int speed = (int) ((Fl_Slider *)o)->value();
  mixer->set_speed(*(int*)ch,speed);
}

void cb_set_position(Fl_Widget *o, void *ch) {
  float pos = ((Fl_Slider *)o)->value();
  if(!mixer->set_position(pos,*(int*)ch))
    warning("can't seek position on channel [%u]",*(int*)ch);
}

void cb_set_playmode(Fl_Widget *o, void *ch) {
  int i = *(int*)ch;

  switch (mixer->xach[i]->playmode)
    {
    case 0: /* LOOP -> PLAY */
      o->label("@--> ONCE");
      mixer->xach[i]->playmode = mixer->xach[i]->PLAY;
      break;
    case 1: /* CONT -> LOOP */
      o->label("@--> LOOP");
      mixer->xach[i]->playmode = mixer->xach[i]->LOOP;
      break;
    case 2: /* PLAY -> CONT */
      o->label("@--> CONT");
      mixer->xach[i]->playmode = mixer->xach[i]->CONT;
      break;
    }

}

void cb_set_icecast(Fl_Widget *o) {

  if(((Fl_Light_Button *)o)->value()==0) { /* stop the stream */
  
    mixer->stop_icecast();
    amuse->status_main->value("Stopped streaming to icecast server");
    amuse->status_ice->value("");

  } else { /* set the stream */
    
    char *ip, *port, *mount, *pass, *name, *url, *desc;
    int bps, login;
    int frequency, lowpass = 0, highpass = 0, mode, quality, nport;

    ip = (char*)amuse->ip_input->value();
    port = (char*)amuse->port_input->value();
    nport = atoi(port);
    
    mount = (char*)amuse->mount_input->value();
    pass = (char*)amuse->pass_input->value();
    bps = amuse->bitrate_choice->value();

    name = (char*)amuse->name_input->value();
    url = (char*)amuse->url_input->value();
    desc = (char*)amuse->description_input->value();

    frequency = amuse->resample_choice->value();
    mode = amuse->channels_choice->value();
    quality = (int)amuse->quality_slide->value();
    
    switch(amuse->filtering_choice->value()) {
    case 0:
      lowpass = 0; highpass = 0;
      break;
    case 1:
      lowpass = -1; highpass = -1;
      break;
    case 2:
      lowpass = atoi(amuse->lowpassfreq_input->value());
      highpass = atoi(amuse->highpassfreq_input->value());
      break;
    }

    bps = solve_bps(bps);
    frequency = solve_frequency(frequency);
    login = amuse->login_choice->value();
    
    mixer->set_lame(bps, frequency, mode, quality, lowpass, highpass);
    if(mixer->set_icecast(ip, nport, pass, mount, bps, name, url, desc, login)) {
      char temp[256];
      amuse->status_main->value("WE GOT MILK !!");
      snprintf(temp,256,"STREAMING ON [ http://%s:%u/%s ]",ip,nport,mount);
      amuse->status_ice->value(temp);
      ((Fl_Light_Button *)o)->set();
      save_conf();
    } else {
      amuse->status_main->value("uhm, no milk :(");
      ((Fl_Light_Button *)o)->clear();
    }
  }

}

void cb_set_talk(Fl_Widget *o) {
  int i;
  float temp_vol[amuse->numch];


  if(((Fl_Light_Button *)o)->value()==1) { 
    /* talk is not active; activate it */  
    for (i = 0; i < amuse->numch; i++) {
      store_vol[i] = amuse->chan[i]->volume_slider->value();
      if(store_vol[i]<TALK_VOLUME)
	temp_vol[i] = 0.0;
      else
	temp_vol[i] = TALK_VOLUME;
      amuse->chan[i]->volume_slider->value(temp_vol[i]);
    }

    mixer->set_all_volumes(&temp_vol[0]);

  } else {   
    /* deactivate talk : bring volumes back up */
    for (i = 0; i < amuse->numch; i++)
      amuse->chan[i]->volume_slider->value(store_vol[i]);
    
    mixer->set_all_volumes(&store_vol[0]);    
  }
}

void cb_set_linein(Fl_Widget *o) {
  bool res;

  res = mixer->set_live(((Fl_Light_Button *)o)->value());

  if(res)
    ((Fl_Light_Button *)o)->set();
  else
    ((Fl_Light_Button *)o)->clear();
}

void cb_set_lineout(Fl_Widget *o) {
  bool res;

  res = mixer->set_lineout(((Fl_Light_Button *)o)->value());
  
  if(res)
    ((Fl_Light_Button *)o)->set();
  else
    ((Fl_Light_Button *)o)->clear();
  
}

void cb_save_file(Fl_Widget *o) {
  const char *filename;

  filename = amuse->filesave_input->value();

  /* TURNING ON */
  if (((Fl_Light_Button *)o)->value() == 1)
    {
      if ((filename[0] == '\0') || (filename[0] ==  ' '))   
	{
	  amuse->status_main->value("you need to type in a filename");
	  amuse->filesave_button->value(0);
	} else {
	  int res, bps, freq, mode, quality, lowpass = 0, highpass = 0;	  

	  /* retrieve lame settings */
	  bps = solve_bps(amuse->bitrate_choice->value());
	  freq = amuse->resample_choice->value();
	  mode = amuse->channels_choice->value();
	  quality = (int) amuse->quality_slide->value();
	  
	  switch(amuse->filtering_choice->value()) {
	  case 0:
	    lowpass = 0; highpass = 0;
	    break;
	  case 1:
	    lowpass = -1; highpass = -1;
	    break;
	  case 2:
	    lowpass = atoi(amuse->lowpassfreq_input->value());
	    highpass = atoi(amuse->highpassfreq_input->value());
	    break;
	  }
	  freq = solve_frequency(freq);
	  
	  mixer->set_lame(bps, freq, mode, quality, lowpass, highpass);

	  /* input syntax ok, let's start recording */
	  res = mixer->set_mp3file((char*)filename, bps);
	  /* if res -1 should notice error */

	  amuse->filebrowse_button->deactivate();
	  amuse->filesave_input->deactivate();
	  amuse->status_main->value(filename);
	}
    }

  /* TURNING OFF */
  if (((Fl_Light_Button *)o)->value() == 0)
    {
      /* let's shutdown file recording */
      mixer->stop_mp3file();

      amuse->filebrowse_button->activate();
      amuse->filesave_input->activate();
      amuse->status_main->value(" ");
      
    }
}

void cb_pick_savefile(Fl_Widget *o) {
    char *filename = fl_file_chooser("Pick a file", "*", NULL);
    if(filename) amuse->filesave_input->value(filename); 
}

void cb_pick_file(Fl_Widget *o, void *ch) {
  char *path, *file, *p;
  int chan = *(int *)ch;

  path = fl_file_chooser("Pick a sound file",
			     "*.{[mM][pP]3|[oO][gG][gG]}", pick_file_dir);
  if(!path) return;

  strncpy(pick_file_dir,path,1023);
  p  = (char*) filename_name(pick_file_dir); *p = '\0';
  file = (char*) filename_name(path);

  if( mixer->add_to_playlist(path,chan) )
    amuse->chan[chan]->playlist_browser->add(file,NULL);
}

void cb_pick_url(Fl_Widget *o, void *ch) {
  char *url;
  int chan = *(int *)ch;
  
  url = (char*)fl_input("type the URL of network stream you want to listen",NULL);
  if(!url) return;

  if(strncmp(url,"http://",7)!=0) {
    amuse->status_main->value("invalid URL! http:// is missing");
    return;
  }
  
  if( mixer->add_to_playlist(url,chan) )
    amuse->chan[chan]->playlist_browser->add(url,NULL);
}

void cb_filtering_choice(Fl_Widget *o) {

  int i = amuse->filtering_choice->value();
  switch(i) {
  case 0:
  case 1:
    amuse->lowpassfreq_input->deactivate();
    amuse->highpassfreq_input->deactivate();
    break;
  case 2:
    amuse->lowpassfreq_input->activate();
    amuse->highpassfreq_input->activate();
    break;
  }
}

void cb_open_info(Fl_Widget *o) {

  if(!amuse->infowindow) {
    char temp[2048];
    char museversion[128];
    char fltkversion[128];
    char lameversion[128];
    char tempfile[128];
    int fd;
    
    snprintf(museversion,128,"%s version %s",PACKAGE,VERSION);
    snprintf(fltkversion,128,"FLTK version %u.%u.%u",
	     FL_MAJOR_VERSION,FL_MINOR_VERSION,FL_PATCH_VERSION);
    snprintf(lameversion,128,"%s",amuse->status_lame->value());

    snprintf(temp,2048,
	     "<html><head><title>%s</title></head>"
	     "<body bgcolor=\"#ffeedd\">"
	     "<h2>%s<br>Multiple Streaming Engine<br>http://muse.dyne.org</h2>"
	     "<h5>authors:</h5>"
	     "<table><tr><td align=center><b>JAROMIL<br><br>coder & mantainer<br><i>jaromil@dyne.org</i></b></td>"
	     "<td align=center>&</td>"
	     "<td align=center><b>AUGUST BLACK<br>graphical interface<br><i>august@alien.mur.at</i></b></td></tr></table>"
	     "<i>realized out of an idea by Markus Seidl aka mexx</i>"

	     "<h6>thanks to:</h6><b>SERVUS.AT, Freaknet Medialab, Hell Voyager, LOA hacklab, BBK, Alex+Rasty+Martinez, Asbesto Molesto, Newmark, Strano Network, autistici.org and all the brothers fighting for freedom of speech.</b>"

	     "<h6>MuSE screams on:</h6><b>"
	     "Indymedia radio - http://radio.indymedia.org<br>"
	     "RadioCybernet - http://kyuzz.org/radiocybernet<br>"
	     "Fundamental Radio - http://funda.ment.org</b>"

	     "<hr><p>dinamically linking:<br>"
	     "- %s available at http://www.mp3dev.org<br>"
#ifdef HAVE_VORBIS
	     "- OggVorbis libraries available at http://www.xiph.org<br>"
#endif
	     "statically linking:<br>"
	     "- the toolkit %s available at http://www.fltk.org<br>"
	     "- the mpeg decoding library LIBMPEG originally written by Woo-jae Jung<br>"
	     "- the LIBSHOUT library available at http://www.icecast.org<br>"
	     "</p><hr>"

	     "<p>MuSE is copyright (C) 2000-2002 Denis Roio aka jaromil<br>"
	     "Amuse fltk GUI is copyright (C) 2000-2001 August Black</p>"

	     "<p>This program is free software; you can redistribute it and/or modify<br>"
	     "it under the terms of the GNU General Public License as published by<br>"
	     "the Free Software Foundation; either version 2 of the License, or<br>"
	     "(at your option) any later version.<br>"
	     "This program is distributed in the hope that it will be useful,<br>"
	     "but WITHOUT ANY WARRANTY; without even the implied warranty of<br>"
	     "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the<br>"
	     "GNU General Public License for more details.<br>"
	     "You should have received a copy of the GNU General Public License<br>"
	     "along with this program; if not, write to the Free Software<br>"
	     "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA</p>"
	     "</body></html>",museversion,museversion,lameversion,fltkversion);

    sprintf(tempfile,"/tmp/.museXXXXXX");
    fd = mkstemp(tempfile);
    if(fd<0) { error("can't create temp file"); return; }
    write(fd,temp,strlen(temp));
    close(fd);
    amuse->infowindow = new Fl_Help_Dialog;
    amuse->infowindow->load(tempfile);
    unlink(tempfile);
  }
  amuse->infowindow->show();

}

void cb_add(Fl_Widget *o){
  //  Amuse *amuse = ((Amuse *)o->parent());
  if (amuse->numch < MAX_CHANNELS) {
    if (mixer->create_channel(amuse->numch)) {

      if (amuse->numch == 0) amuse->talk_button->show();
      amuse->chan[amuse->numch]->show();
      amuse->add(amuse->chan[amuse->numch]);
      amuse->numch++;
      if (amuse->col < amuse->colwidth) {
	amuse->col++;
      } else {  
	amuse->row++;
	amuse->col = 1; 
      }
      amuse->reshape();
      amuse->redraw(); 
    }
  }
}

void cb_remove(Fl_Widget *o){
  if (amuse->numch > 0) {
    amuse->numch--; 
    mixer->delete_channel(amuse->numch);
    amuse->chan[amuse->numch]->hide();     

    amuse->col--;
    if (amuse->col <= 0) {
      amuse->row--;
      amuse->col = amuse->colwidth; 
    }
    amuse->reshape();
    amuse->redraw(); 
  } 
}

void cb_view(Fl_Widget *o) {
  if (amuse->conf_win->visible() == false)
    amuse->conf_win->show();
  else amuse->conf_win->hide();
}
void cb_levels(Fl_Widget *o) {
  if (amuse->levels_win->visible() == false) {
    amuse->levels->showit = true;
    amuse->levels_win->show();
  }  else {
    amuse->levels->showit = false;
    amuse->levels_win->hide();
  }
}
void gui_quit(Fl_Menu_* o, void* v) {
  amuse->quit = true;
  mixer->quit = true;
}
